@@ -26,7 +26,7 @@ from isisdl.settings import download_timeout, download_timeout_multiplier, downl
     extern_ignore, log_file_location, datetime_str
 from isisdl.settings import enable_multithread, discover_num_threads, is_windows, is_macos, is_testing, testing_bad_urls, url_finder, isis_ignore
 from isisdl.utils import User, path, sanitize_name, args, on_kill, database_helper, config, generate_error_message, logger, parse_google_drive_url, get_url_from_gdrive_confirmation, \
-    DownloadThrottler, MediaType, HumanBytes
+    DownloadThrottler, MediaType, HumanBytes, normalize_url
 from isisdl.utils import calculate_local_checksum
 from isisdl.version import __version__
 
@@ -154,10 +154,7 @@ class PreMediaContainer:
         if config.make_subdirs is False:
             relative_location = ""
 
-        if url.endswith("?forcedownload=1"):
-            url = url[:-len("?forcedownload=1")]
-
-        self.url = url
+        self.url = normalize_url(url)
         self._name = name
         self.time = time
         self.size = size
@@ -409,6 +406,21 @@ class MediaContainer:
 
         return calculate_local_checksum(self.path) == maybe_container.checksum
 
+    def hardlink(self, other: MediaContainer) -> None:
+        # TODO: Remove
+        if is_testing:
+            assert self.should_download
+
+        self.current_size = other.current_size
+        self.media_type = other.media_type
+        self.checksum = other.checksum
+
+        self.path.unlink(missing_ok=True)
+        os.link(other.path, self.path)
+
+        self.dump()
+        self._done = True
+
     def dump(self) -> MediaContainer:
         database_helper.add_pre_container(self)
         return self
@@ -476,7 +488,14 @@ class MediaContainer:
             self.current_size = None
             self.media_type = MediaType.corrupted
             self._done = True
-            database_helper.add_bad_url(self.url)
+            if self.media_type != MediaType.video:
+                # The video server is sometimes unreliable but it _should_ always work. So don't add these url's
+                database_helper.add_bad_url(self.url)
+
+            for link in self._links:
+                if link.should_download:
+                    link.hardlink(self)
+
             self.dump()
             return
 
@@ -528,14 +547,8 @@ class MediaContainer:
                 assert link.size == self.size
                 assert link._links == []
 
-            link.current_size = self.current_size
-            link.media_type = self.media_type
-            link.checksum = self.checksum
-
-            link.path.unlink(missing_ok=True)
-            os.link(self.path, link.path)
-            link.dump()
-            link._done = True
+            if link.should_download:
+                link.hardlink(self)
 
         self._newly_downloaded = True
         self._done = True
@@ -588,6 +601,7 @@ class Course:
 
         content = cast(List[Dict[str, Any]], content)
         all_content: List[PreMediaContainer] = []
+        parsed_url_ids = set()
 
         for week in content:
             if "modules" not in week:
@@ -595,13 +609,7 @@ class Course:
 
             module: Dict[str, Any]
             for module in week["modules"]:
-                # Check if the description contains url's to be followed
-                if "description" in module:
-                    links = url_finder.findall(module["description"])
-                    for link in links:
-                        parse = urlparse(link)
-                        if parse.scheme and parse.netloc and config.follow_links and extern_ignore.match(link) is None and isis_ignore.match(link) is None:
-                            all_content.append(PreMediaContainer(link, self, MediaType.extern, None))
+                parsed_url_ids.add(str(module["id"]))
 
                 if "url" not in module:
                     continue
@@ -625,12 +633,28 @@ class Course:
 
                 if "contents" in module:
                     for file in module["contents"]:
-                        if config.follow_links and "type" in file and file["type"] == "url" and "fileurl" in file and extern_ignore.match(file["fileurl"]) is None and isis_ignore.match(
-                                file["fileurl"]) is None:
+                        if config.follow_links and "type" in file and file["type"] == "url" and "fileurl" in file and extern_ignore.match(file["fileurl"]) is None \
+                                and isis_ignore.match(file["fileurl"]) is None:
                             all_content.append(PreMediaContainer(file["fileurl"], self, MediaType.extern))
                         else:
                             all_content.append(PreMediaContainer(file["fileurl"], self, MediaType.document, file["filename"], file["filepath"], file["filesize"], file["timemodified"]))
 
+        if config.follow_links is False:
+            return all_content
+
+        # Now check all links to find those that were not parsed yet
+        links = {normalize_url(url) for url in url_finder.findall(str(content))} - {item.url for item in all_content} - parsed_url_ids
+        _links = []
+        for link in links:
+            possible_id = re.findall(r"https://isis\.tu-berlin\.de/.*/view\.php\?id=(\d*).*", link)
+            if possible_id is not None and len(possible_id) == 1 and possible_id[0] in parsed_url_ids:
+                continue
+
+            parse = urlparse(link)
+            if parse.scheme and parse.netloc and extern_ignore.match(link) is None and isis_ignore.match(link) is None:
+                all_content.append(PreMediaContainer(link, self, MediaType.extern, None))
+                _links.append(link)
+
         return all_content
 
     def path(self, *args: str) -> Path:
@@ -769,6 +793,14 @@ class RequestHelper:
 
         return response.json()
 
+    @staticmethod
+    def analyze_most_common_urls(pre_containers: List[PreMediaContainer]) -> None:
+        urls: DefaultDict[Optional[str], int] = defaultdict(int)
+        for container in pre_containers:
+            urls[urlparse(container.url).hostname] += 1
+
+        pass
+
     def download_content(self, status: Optional[RequestHelperStatus] = None) -> Dict[MediaType, List[MediaContainer]]:
         if status is not None:
             status.set_total(len(self.courses))
@@ -790,6 +822,8 @@ class RequestHelper:
         pre_containers = list({f"{item.course} {item.url}": item for item in pre_containers}.values())
         random.shuffle(pre_containers)
 
+        self.analyze_most_common_urls(pre_containers)
+
         if (num_cached := sum(pre_container.is_cached for pre_container in pre_containers)) != len(pre_containers):
             if status is not None:
                 status.set_total(len([item for item in pre_containers if not item.is_ready]))
@@ -913,13 +947,16 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
     for file in {file.path: file for file in files}.values():
         hard_link_conflicts[f"{file.course.course_id} {file._name} {file.size}"].append(file)
 
+    new_files = []
     for conflict in hard_link_conflicts.values():
-        if len(conflict) != 1:
+        if len(conflict) > 1:
             conflict.sort(key=lambda x: x.time)
             conflict[0]._links.extend(conflict[1:])
             final_list.append(conflict[0])
-            for item in conflict[0]._links:
-                files.remove(item)
+        else:
+            new_files.append(conflict[0])
+
+    files = new_files
 
     conflicts: DefaultDict[Path, List[MediaContainer]] = defaultdict(list)
     for file in set(files):
@@ -929,10 +966,11 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
         conflict.sort(key=lambda x: x.time)
 
         if len(conflict) == 1:
-            # Only same sizes
-            final_list.append(conflict[0])
+            # Don't handle the conflict here
+            continue
 
         elif all(item.size == conflict[0].size for item in conflict):
+            # Only same sizes
             final_list.append(conflict[0])
 
         elif len(set(item.size for item in conflict)) == len(conflict):
@@ -948,8 +986,6 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
             logger.assert_fail(f"conflict: {[{x: getattr(item, x) for x in item.__slots__} for item in conflict]}")
             continue
 
-    # TODO: This takes too long.
-
     # Finally filter the remaining files based on the url
     hard_link_conflicts = defaultdict(list)
 
@@ -957,20 +993,18 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
         hard_link_conflicts[file.download_url].append(file)
 
     conflict_urls: Set[str] = set()
-    new_files = []
 
     for conflict in hard_link_conflicts.values():
-        if len(conflict) != 1:
+        if len(conflict) > 1:
             conflict.sort(key=lambda x: x.time)
             conflict[0]._links.extend(conflict[1:])
             conflict_urls.add(conflict[0].url)
-            new_files.append(conflict[0])
-
             final_list.append(conflict[0])
-            for item in conflict[0]._links:
-                final_list.remove(item)
 
-    return [file for file in final_list if file.url not in conflict_urls] + new_files
+        else:
+            final_list.append(conflict[0])
+
+    return [file for file in final_list if file.url not in conflict_urls]
 
 
 class Downloader(Thread):
@@ -1052,6 +1086,10 @@ class CourseDownloader:
             for container in collapsed_containers:
                 if container.should_download:
                     container.path.open("w").close()
+                else:
+                    for con in container._links:
+                        if con.should_download:
+                            con.hardlink(container)
 
         CourseDownloader.containers = containers
 
