@@ -65,8 +65,8 @@ def get_args() -> argparse.Namespace:
     operations.add_argument("--config", help="Guides you through additional configuration which focuses on what to download from ISIS.", action="store_true")
     operations.add_argument("--sync", help="Do a full reset of the database, updating all file locations and urls.", action="store_true")
     operations.add_argument("--compress", help="Starts ffmpeg and will compress all downloaded videos.", action="store_true")
-    operations.add_argument("--subscribe", help="Subscribes you to *all* ISIS courses publicly available.", action="store_true")
-    operations.add_argument("--unsubscribe", help="Unsubscribes you from the courses you got subscribed by running `isisdl --subscribe`.", action="store_true")
+    # operations.add_argument("--subscribe", help="Subscribes you to *all* ISIS courses publicly available.", action="store_true")
+    # operations.add_argument("--unsubscribe", help="Unsubscribes you from the courses you got subscribed by running `isisdl --subscribe`.", action="store_true")
     if not is_windows:
         operations.add_argument("--export-config", help=f"Exports the config to {export_config_file_location}.", action="store_true")
 
@@ -677,6 +677,13 @@ def path(*args: str) -> Path:
     return Path(working_dir_location, *args)
 
 
+def normalize_url(url: str) -> str:
+    if url.endswith("?forcedownload=1"):
+        url = url[:-len("?forcedownload=1")]
+
+    return url
+
+
 def remove_systemd_timer() -> None:
     if not os.path.exists(systemd_timer_file_location):
         return
@@ -1159,7 +1166,11 @@ class DataLogger(Thread):
         try:
             while True:
                 item = self.messages.get()
-                self.s.post("http://49.12.42.246/isisdl/", json=item)
+                try:
+                    self.s.post("http://49.12.42.246/isisdl/", json=item)
+                except Exception:
+                    return
+
                 self.done.put(None)
         except Exception as ex:
             generate_error_message(ex)
